from collections import defaultdict, deque

def find_protected_warehouses(n, m, k, roads, guards):
    # Построение графа
    graph = defaultdict(list)
    for a, b in roads:
        graph[a].append(b)
        graph[b].append(a)

    # Подготовка для многократного BFS
    queue = deque()
    distance = {}  # Минимальное расстояние до каждого склада
    for p, h in guards:
        queue.append((p, h))  # Добавляем склад и его радиус
        distance[p] = max(distance.get(p, 0), h)  # Учёт наибольшей выносливости

    # BFS
    while queue:
        current, remaining = queue.popleft()
        if remaining <= 0:
            continue
        for neighbor in graph[current]:
            if neighbor not in distance or distance[neighbor] < remaining - 1:
                distance[neighbor] = remaining - 1
                queue.append((neighbor, remaining - 1))

    # Результат
    protected = sorted(distance.keys())
    return len(protected), protected


# Чтение входных данных
n, m, k = map(int, input().split())  # Количество складов, дорог, охранников
roads = [tuple(map(int, input().split())) for _ in range(m)]  # Дороги между складами
guards = [tuple(map(int, input().split())) for _ in range(k)]  # Охранники и их выносливость

# Решение задачи
count, protected_warehouses = find_protected_warehouses(n, m, k, roads, guards)

# Вывод результата
print(count)
print(" ".join(map(str, protected_warehouses)))
