from collections import defaultdict, deque

def find_protected_warehouses(n, m, k, roads, guards):
    # Построение графа
    graph = defaultdict(list)
    for a, b in roads:
        graph[a].append(b)
        graph[b].append(a)

    # Если нет охранников, результат пустой
    if k == 0:
        return 0, []

    # Очередь для BFS и контроль посещённых вершин
    queue = deque()
    max_reach = {}  # Хранит максимальную досягаемость для каждого склада
    visited = set()

    # Добавляем охранников в очередь
    for p, h in guards:
        queue.append((p, h))
        max_reach[p] = max(max_reach.get(p, 0), h)
        visited.add((p, h))

    # BFS
    while queue:
        current, remaining = queue.popleft()
        if remaining <= 0:
            continue
        for neighbor in graph[current]:
            if (neighbor not in max_reach or max_reach[neighbor] < remaining - 1):
                max_reach[neighbor] = remaining - 1
                queue.append((neighbor, remaining - 1))

    # Результат
    protected = sorted(max_reach.keys())
    return len(protected), protected


# Чтение входных данных
n, m, k = map(int, input().split())
roads = [tuple(map(int, input().split())) for _ in range(m)]
guards = [tuple(map(int, input().split())) for _ in range(k)]

# Решение задачи
count, protected_warehouses = find_protected_warehouses(n, m, k, roads, guards)

# Вывод результата
print(count)
print(" ".join(map(str, protected_warehouses)))
